// ---------------------------------------------------------------------------
// - Main.cpp                                                                -
// - the afnix interpreter main program                                      -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2006 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Interp.hpp"
#include "System.hpp"
#include "Options.hpp"

// main interpreter 

int main (int argc, char** argv) {
  using namespace afnix;

  // the error stream
  OutputTerm terr (OutputTerm::ERROR);

  try {
    // create a new option instance
    Options* options = new Options (argc,argv);

    // get the optional file name
    String fname = options->getfname ();
    bool   sflag = (fname.length () == 0);
    
    // create a new interpreter
    Input*  is = nilp;
    Output* os = nilp;
    Output* es = nilp;
    Interp* interp = (sflag == true) ? new Interp : new Interp (is,os,es);
    
    // eventually add the initial path
    if ((sflag == false) && (options->getnoipth () == false)) {
      interp->addpath (System::xdir (fname));
    }
    
    // set some interpreter flags
    interp->setassert (options->getassert ());
    interp->setargs   (options->getargs   ());
    interp->setpath   (options->getpath   ());
    
    // loop or execute on the standard input or a file
    bool status  = (sflag == true) ? interp->loop () : interp->loop (fname); 
    // clean eveything and return
    delete options;
    delete interp;
    if (status == false) return 1;
  } catch (const Exception& e) {
    terr.errorln (e);
    return 2;
  } catch (...) {
    terr << "fatal: unknown exception trapped\n";
    return -1;
  }
  // here everythin went fine
  return 0;
}
