// ---------------------------------------------------------------------------
// - cloc.cxx                                                                -
// - standard system library - c localization function implementation        -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2006 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "cstr.hpp"
#include "csys.hpp"
#include "cloc.hpp"
#include "cloc.hxx"

namespace afnix {

  // system wide localization initialization

  void c_iloc (void) {
    // set the system locale
    setlocale (LC_ALL, NULL);
  }

  // get the system codeset

  t_codeset c_getcset (void) {
    // map with the global locale
    const char* locale = c_getenv ("LC_ALL");
    if (c_strlen (locale) > 0) return c_mapcset (locale);
    // map with the character type
    locale = c_getenv ("LC_CTYPE");
    if (c_strlen (locale) > 0) return c_mapcset (locale);
    // map with the language type
    locale = c_getenv ("LANG");
    return c_mapcset (locale);    
  }

  // map the codeset by string

  t_codeset c_mapcset (const char* locale) {
    // check for nil
    if (locale == nilp) return LOC_ASCII;
    // start by a fresh copy
    char* lptr = c_strdup (locale);
    char* cptr = lptr;
    // try to find a dot
    while (*cptr++ != nilc) {
      if (*cptr == '.') break;
    }
    // if we are at the end return in the ascii locale
    if (*cptr == nilc) return LOC_ASCII;
    // here we have a dot - double check and advance
    if (*cptr++ != '.') return LOC_ASCII;
    // eventually find a modifier
    char* cend = cptr;
    while (*cend++ != nilc) {
      if (*cend == '@') {
	*cend = nilc;
	break;
      }
    }
    // map the resulting string in upper case
    char* codeset = c_toupper (cptr);
    delete [] lptr;
    // map the ascii codeset
    if (c_strcmp (codeset, "C")        ||
	c_strcmp (codeset, "POSIX")    ||
	c_strcmp (codeset, "US-ASCII")) {
      delete [] codeset;
      return LOC_ASCII;
    }
    // map the iso codeset codeset
    if (c_strcmp (codeset, "ISO-8859-1")  ||
	c_strcmp (codeset, "ISO-8859-15")) {
      delete [] codeset;
      return LOC_LATIN;
    }
    // map the unicode codeset
    if (c_strcmp (codeset, "UTF8")  ||
	c_strcmp (codeset, "UTF-8")) {
      delete [] codeset;
      return LOC_UTF8;
    }
    // by default map to ascii
    delete [] codeset;
    return LOC_ASCII;
  }
}
