// ---------------------------------------------------------------------------
// - Iterator.cpp                                                            -
// - standard object library - iterator abstract class implementation        -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2006 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "String.hpp"
#include "Vector.hpp"
#include "Boolean.hpp"
#include "Iterator.hpp"

namespace afnix {
  
  // the iterator supported quarks
  static const long QUARK_END      = String::intern ("end");
  static const long QUARK_ENDP     = String::intern ("end-p");
  static const long QUARK_NEXT     = String::intern ("next");
  static const long QUARK_BEGIN    = String::intern ("begin");
  static const long QUARK_VALIDP   = String::intern ("valid-p");
  static const long QUARK_GETOBJ   = String::intern ("get-object");
  static const long QUARK_PREVIOUS = String::intern ("previous");

  // the quark table info
  static const long QUARK_TABLE_LEN = 7;
  static const long QUARK_TABLE_MAP [QUARK_TABLE_LEN] = {
    QUARK_END,
    QUARK_ENDP,
    QUARK_NEXT,
    QUARK_BEGIN,
    QUARK_VALIDP,
    QUARK_GETOBJ,
    QUARK_PREVIOUS
  };

  bool Iterator::valid (void) {
    rdlock ();
    bool result = (isend () == true) ? false : true;
    unlock ();
    return result;
  }

  // return true if the given quark is defined

  bool Iterator::isquark (const long quark, const bool hflg) const {
    rdlock ();
    for (long i = 0; i < QUARK_TABLE_LEN; i++) {
      if (quark == QUARK_TABLE_MAP[i]) {
	unlock ();
	return true;
      }      
    }
    bool result = hflg ? Object::isquark (quark, hflg) : false;
    unlock ();
    return result;
  }

  // apply an iterator method with a set of arguments by quark

  Object* Iterator::apply (Runnable* robj, Nameset* nset, const long quark,
			   Vector* argv) {

    // get the number of arguments
    long argc = (argv == nilp) ? 0 : argv->length ();

    // dispatch 0 argument
    if (argc == 0) {
      if (quark == QUARK_GETOBJ) return getobj ();
      if (quark == QUARK_VALIDP) return new Boolean (valid ());
      if (quark == QUARK_ENDP)   return new Boolean (isend ());
      if (quark == QUARK_BEGIN) {
	begin ();
	return nilp;
      }
      if (quark == QUARK_END) {
	end ();
	return nilp;
      }
      if (quark == QUARK_NEXT) {
	next ();
	return nilp;
      }
      if (quark == QUARK_PREVIOUS) {
	prev ();
	return nilp;
      }
    }

    // call the object method
    return Object::apply (robj, nset, quark, argv);
  }
}
