// ---------------------------------------------------------------------------
// - Stream.hpp                                                              -
// - standard object library - base stream class definition                  -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2006 amaury darsch                                   -
// ---------------------------------------------------------------------------

#ifndef  AFNIX_STREAM_HPP
#define  AFNIX_STREAM_HPP

#ifndef  AFNIX_STRING_HPP
#include "String.hpp"
#endif

namespace afnix {

  /// The stream class is a base class which defines the stream operating mode.
  /// BY default, the stream operates in byte mode but can be set to operate
  /// in UTF-8 mode when unicode character are used. When operating in byte
  /// mode, only latin (aka 8 bits) characters are allowed. When operating
  /// in other mode, the stream might read or write several bytes before 
  /// accepting or displaying a character.
  /// @author amaury darsch

  class Stream : public virtual Object {
  public:
    /// the stream encoding types
    enum t_smod {BYTE, UTF8};

    /// @return the system stream mode
    static t_smod getssm (void);

  protected:
    /// the stream coding code
    t_smod d_smod;

  public:
    /// create a default byte stream
    Stream (void);

    /// create a stream by coding mode
    /// @param smod the stream coding mode to set
    Stream (const t_smod smod);

    /// @return the stream coding mode
    t_smod getsmod (void) const;

    /// set the stream coding mode
    /// @param smod the stream coding mode to set
    void setsmod (const t_smod smod);

    /// evaluate a stream data member
    /// @param robj  robj the current runnable
    /// @param nset  the current nameset
    /// @param quark the quark to evaluate
    static Object* meval (Runnable* robj, Nameset* nset, const long quark);

    /// @return true if the given quark is defined
    bool isquark (const long quark, const bool hflg) const;

    /// apply this input with a set of arguments and a quark
    /// @param robj   robj the current runnable
    /// @param nset   the current nameset    
    /// @param quark  the quark to apply these arguments
    /// @param argv   the arguments  to apply
    Object* apply (Runnable* robj, Nameset* nset, const long quark,
		   Vector* argv);
  };
}

#endif
